@@ -1,9 +1,13 @@
-import math
+
+from typing import Tuple
+
+from sqlalchemy.sql import func
 import pandas as pd
-import pickle as pkl
+
 from ..wrapper.mysql import BasicDatabaseConnector
 from ..view.basic_models import *
 
+
 class BasicDataApi(object):
     def get_trading_day_list(self):
         with BasicDatabaseConnector().managed_session() as db_session:
@@ -13,7 +17,7 @@ class BasicDataApi(object):
                     )
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
-            
+
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))
 
@@ -25,11 +29,10 @@ class BasicDataApi(object):
                     )
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
-            
+
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))
 
-
     def get_index_info(self):
         with BasicDatabaseConnector().managed_session() as db_session:
             try:
@@ -38,9 +41,9 @@ class BasicDataApi(object):
                     )
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
-            
+
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))            
+                print('Failed to get data <err_msg> {}'.format(e))
 
     def get_stock_info(self):
         with BasicDatabaseConnector().managed_session() as db_session:
@@ -50,9 +53,9 @@ class BasicDataApi(object):
                     )
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
-            
+
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))            
+                print('Failed to get data <err_msg> {}'.format(e))
 
     def get_fund_nav(self, fund_list):
         with BasicDatabaseConnector().managed_session() as db_session:
@@ -63,32 +66,47 @@ class BasicDataApi(object):
                         # We could query all fund_ids at one time
                         FundNav.fund_id.in_(fund_list),
                     )
-                    
+
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
-            
-            except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))            
 
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+
+    def delete_fund_nav(self, start_date, end_date):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                        FundNav
+                ).filter(    
+                    FundNav.datetime >= start_date,
+                    FundNav.datetime <= end_date,
+                ).delete(synchronize_session=False)
+                db_session.commit()
+                return True
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
+                
     def get_fund_nav_with_date(self, start_date, end_date, fund_list):
         with BasicDatabaseConnector().managed_session() as db_session:
             try:
                 query = db_session.query(
                         FundNav.fund_id,
-                        FundNav.adjusted_net_value, 
+                        FundNav.adjusted_net_value,
                         FundNav.datetime
                     ).filter(
                         FundNav.fund_id.in_(fund_list),
                         FundNav.datetime >= start_date,
                         FundNav.datetime <= end_date,
                     )
-                    
+
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
-            
+
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))        
-                
+                print('Failed to get data <err_msg> {}'.format(e))
+
     def get_stock_price(self, stock_list):
         with BasicDatabaseConnector().managed_session() as db_session:
             try:
@@ -98,12 +116,12 @@ class BasicDataApi(object):
                         # We could query all fund_ids at one time
                         StockPrice.stock_id.in_(stock_list),
                     )
-                    
+
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
-            
+
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))            
+                print('Failed to get data <err_msg> {}'.format(e))
 
 
     def get_fund_ret(self, fund_list):
@@ -115,12 +133,12 @@ class BasicDataApi(object):
                         # We could query all fund_ids at one time
                         FundRet.fund_id.in_(fund_list),
                     )
-                    
+
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
-            
+
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))            
+                print('Failed to get data <err_msg> {}'.format(e))
 
 
     def get_index_price(self, index_list):
@@ -132,12 +150,12 @@ class BasicDataApi(object):
                         # We could query all fund_ids at one time
                         IndexPrice.index_id.in_(index_list),
                     )
-                    
+
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
-            
+
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))            
+                print('Failed to get data <err_msg> {}'.format(e))
 
     def delete_index_price(self, index_id_list, start_date, end_date):
         with BasicDatabaseConnector().managed_session() as db_session:
@@ -167,12 +185,12 @@ class BasicDataApi(object):
                         FundNav.datetime >= start_date,
                         FundNav.datetime <= end_date,
                     )
-                    
+
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
-            
+
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))   
+                print('Failed to get data <err_msg> {}'.format(e))
 
 
     def get_index_benchmark_data(self, index_list, start_date, end_date):
@@ -188,12 +206,12 @@ class BasicDataApi(object):
                         IndexPrice.datetime >= start_date,
                         IndexPrice.datetime <= end_date,
                     )
-                    
+
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
-            
+
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))   
+                print('Failed to get data <err_msg> {}'.format(e))
 
 
     def get_fund_list(self):
@@ -204,7 +222,7 @@ class BasicDataApi(object):
                     )
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
-            
+
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))
 
@@ -218,7 +236,7 @@ class BasicDataApi(object):
                     )
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
-            
+
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))
 
@@ -231,7 +249,7 @@ class BasicDataApi(object):
                     )
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
-            
+
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))
 
@@ -242,7 +260,7 @@ class BasicDataApi(object):
                 query_results = db_session.query(
                         FundInfo.fund_id,
                         FundInfo.order_book_id,
-                        FundInfo.start_date, 
+                        FundInfo.start_date,
                         FundInfo.end_date
                     ).all()
 
@@ -255,7 +273,7 @@ class BasicDataApi(object):
             except Exception as e:
                 print('Failed to get fund id mapping <err_msg> {}'.format(e))
                 return None
-                
+
         return fund_id_mapping
 
     def get_fund_size(self):
@@ -266,6 +284,37 @@ class BasicDataApi(object):
                     )
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
-            
+
             except Exception as e:
-                print('Failed to get data <err_msg> {}'.format(e))
\ No newline at end of file
+                print('Failed to get data <err_msg> {}'.format(e))
+
+    def get_style_analysis_data(self, start_date: str, end_date: str, stock_list: Tuple[str] = ()):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    StyleAnalysisStockFactor
+                )
+                if stock_list:
+                    query = query.filter(
+                        StyleAnalysisStockFactor.stock_id.in_(stock_list)
+                    )
+                query = query.filter(
+                    StyleAnalysisStockFactor.datetime >= start_date,
+                    StyleAnalysisStockFactor.datetime <= end_date,
+                )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {StyleAnalysisStockFactor.__tablename__}')
+                return
+
+    def get_style_analysis_time_range(self) -> pd.DataFrame:
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    func.max(StyleAnalysisStockFactor.datetime).label('end_date'),
+                    func.min(StyleAnalysisStockFactor.datetime).label('start_date'),
+                )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get time range data <err_msg> {e} from {StyleAnalysisStockFactor.__tablename__}')
+                return
