@@ -1,9 +1,13 @@
+
+from typing import Tuple
+
 import math
 import pandas as pd
 import pickle as pkl
 from ..wrapper.mysql import DerivedDatabaseConnector
 from ..view.derived_models import *
 
+
 class DerivedDataApi(object):
     def get_fund_indicator(self, fund_list):
         with DerivedDatabaseConnector().managed_session() as quant_session:
@@ -23,6 +27,21 @@ class DerivedDataApi(object):
             
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))
+
+    def get_fund_indicator_by_date(self, fund_list, date):
+        with DerivedDatabaseConnector().managed_session() as quant_session:
+            try:
+                query = quant_session.query(
+                    FundIndicator
+                ).filter(
+                    FundIndicator.datetime == date,
+                    FundIndicator.fund_id.in_(fund_list),
+                )
+                tag_df = pd.read_sql(query.statement, query.session.bind)
+                return tag_df
+
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
         
     def get_fund_score(self):
         with DerivedDatabaseConnector().managed_session() as quant_session:
@@ -93,4 +112,36 @@ class DerivedDataApi(object):
                 return df
             except Exception as e:
                 print(e)
-                return pd.DataFrame([])
\ No newline at end of file
+                return pd.DataFrame([])
+
+    def get_style_factor_return(self, start_date: str, end_date: str, index_list: Tuple[str] = ()):
+        with DerivedDatabaseConnector().managed_session() as session:
+            try:
+                query = session.query(
+                    StyleAnalysisFactorReturn
+                )
+                if index_list:
+                    query = query.filter(StyleAnalysisFactorReturn.universe_index.in_(index_list))
+                query = query.filter(
+                    StyleAnalysisFactorReturn.datetime >= start_date,
+                    StyleAnalysisFactorReturn.datetime <= end_date,
+                )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print('Failed to get_style_factor_return <err_msg> {}'.format(e))
+                return
+
+    def get_allocation_distribution(self, version:str=1):
+        with DerivedDatabaseConnector().managed_session() as session:
+            try:
+                query = session.query(
+                    AllocationDistribution
+                ).filter(
+                    AllocationDistribution.version == version
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+            except Exception as e:
+                print('Failed to get_style_factor_return <err_msg> {}'.format(e))
+                return
+
