@@ -253,6 +253,21 @@ class RawDataApi(object):
                 print('Failed to get data <err_msg> {}'.format(e))
                 return None
 
+    def delete_em_fund_nav(self, start_date, end_date):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EmFundNav
+                ).filter(
+                    EmFundNav.DATES >= start_date,
+                    EmFundNav.DATES <= end_date
+                ).delete(synchronize_session=False)
+                db_session.commit()
+                return True
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
+
     def get_rq_fund_size(self):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
@@ -358,12 +373,15 @@ class RawDataApi(object):
                 print('Failed to get data <err_msg> {}'.format(e))
                 return None
 
-    def get_em_stock_price(self, start_date: str, end_date: str, stock_list: Tuple[str] = ()):
+    def get_em_stock_price(self, start_date: str, end_date: str, stock_list: Tuple[str] = (), columns: Tuple[str] = ()):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
                 query = db_session.query(
-                    EmStockPrice
+                    EmStockPrice.CODES,
+                    EmStockPrice.DATES,
                 )
+                if columns:
+                    query = query.add_columns(*columns)
                 if stock_list:
                     query = query.filter(
                         EmStockPrice.CODES.in_(stock_list)
@@ -377,6 +395,28 @@ class RawDataApi(object):
                 print(f'Failed to get data <err_msg> {e} from {EmStockPrice.__tablename__}')
                 return None
 
+    def get_em_stock_post_price(self, start_date: str, end_date: str, stock_list: Tuple[str] = (), columns: Tuple[str] = ()):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EmStockPostPrice.CODES,
+                    EmStockPostPrice.DATES,
+                )
+                if columns:
+                    query = query.add_columns(*columns)
+                if stock_list:
+                    query = query.filter(
+                        EmStockPostPrice.CODES.in_(stock_list)
+                    )
+                query = query.filter(
+                    EmStockPostPrice.DATES >= start_date,
+                    EmStockPostPrice.DATES <= end_date,
+                )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {EmStockPostPrice.__tablename__}')
+                return None
+
     def get_em_stock_info(self, stock_list: Tuple[str] = ()):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
@@ -392,20 +432,23 @@ class RawDataApi(object):
                 print(f'Failed to get data <err_msg> {e} from {EmStockInfo.__tablename__}')
                 return None
 
-    def get_em_daily_info(self, stock_list: Tuple[str] = (), date_list: Tuple[str] = ()):
+    def get_em_daily_info(self, start_date: str, end_date: str, stock_list: Tuple[str] = (), columns: Tuple[str] = ()):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
                 query = db_session.query(
-                    EmStockDailyInfo
+                    EmStockDailyInfo.CODES,
+                    EmStockDailyInfo.DATES,
                 )
+                if columns:
+                    query = query.add_columns(*columns)
                 if stock_list:
                     query = query.filter(
                         EmStockDailyInfo.CODES.in_(stock_list)
                     )
-                if date_list:
-                    query = query.filter(
-                        EmStockDailyInfo.DATES.in_(date_list)
-                    )
+                query = query.filter(
+                    EmStockDailyInfo.DATES >= start_date,
+                    EmStockDailyInfo.DATES <= end_date,
+                )
                 return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
                 print(f'Failed to get data <err_msg> {e} from {EmStockDailyInfo.__tablename__}')
@@ -429,3 +472,36 @@ class RawDataApi(object):
             except Exception as e:
                 print(f'Failed to get data <err_msg> {e} from {EmStockFinFac.__tablename__}')
                 return None
+
+    def get_em_index_component(self, start_date: str, end_date: str, index_list: Tuple[str] = ()):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EmIndexComponent
+                )
+                if index_list:
+                    query = query.filter(
+                        EmIndexComponent.index_id.in_(index_list)
+                    )
+                query = query.filter(
+                    EmIndexComponent.datetime >= start_date,
+                    EmIndexComponent.datetime <= end_date,
+                )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {EmIndexComponent.__tablename__}')
+                return None
+
+    def get_em_fund_holding_rate(self, start_date: str):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EmFundHoldingRate
+                ).filter(
+                    EmFundHoldingRate.DATES >= start_date,
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
