@@ -1,43 +1,20 @@
+
 import pandas as pd
 import traceback
 import datetime
-from ...wrapper.mysql import BasicDatabaseConnector
 from ...api.raw import RawDataApi
 from ...api.basic import BasicDataApi
 from ...view.basic_models import *
+from .basic_data_helper import BasicDataHelper
+
 
 class BasicDataPart1(object):
 
-    def __init__(self):
-        self._index_info_df = BasicDataApi().get_index_info()
-        self._fund_info_df = BasicDataApi().get_fund_info()
-        # Get the mapping between order_book_id and fund_id (which is defined in this system)
-        self._fund_id_mapping = BasicDataApi().get_fund_id_mapping()
-        self.updated_count = {}
-
-    def _upload_basic(self, df, table_name, to_truncate=False):
-        print(table_name)
-        # print(df)
-        if to_truncate:
-            with BasicDatabaseConnector().managed_session() as mn_session:
-                try:
-                    mn_session.execute(f'TRUNCATE TABLE {table_name}')
-                    mn_session.commit()
-                except Exception as e:
-                    print(f'Failed to truncate table {table_name} <err_msg> {e}')
-        df.to_sql(table_name, BasicDatabaseConnector().get_engine(), index=False, if_exists='append')
-
-        if table_name not in self.updated_count:
-            self.updated_count[table_name] = 0
-        self.updated_count[table_name] += df.shape[0]
-
-    def _get_fund_id_from_order_book_id(self, order_book_id, date):
-        if order_book_id in self._fund_id_mapping:
-            for fund_info in self._fund_id_mapping[order_book_id]:
-                if (date >= fund_info['start_date'] and
-                        (fund_info['end_date'] == '00000000' or date <= fund_info['end_date'])):
-                    return fund_info['fund_id']
-        return None
+    def __init__(self, data_helper: BasicDataHelper):
+        self._data_helper = data_helper
+        self._basic_data_api = BasicDataApi()
+        self._index_info_df = self._basic_data_api.get_index_info()
+        self._fund_info_df = self._basic_data_api.get_fund_info()
 
     def _status_mapping(self, x):
         if x == 'Open':
@@ -106,30 +83,31 @@ class BasicDataPart1(object):
         wind_fund_info = wind_fund_info.join(track_index_df[['track_index']])
         wind_fund_info['wind_id'] = wind_fund_info.index
         wind_fund_info['track_index'] = wind_fund_info['track_index'].map(lambda x: dic.get(x,'none'))
-        self._upload_basic(wind_fund_info, FundInfo.__table__.name)
+        self._data_helper._upload_basic(wind_fund_info, FundInfo.__table__.name)
 
     def index_info(self):
         # Update manually
         # Not verified
         df = pd.read_csv('./data/index_info.csv')
-        self._upload_basic(df, IndexInfo.__table__.name)
+        self._data_helper._upload_basic(df, IndexInfo.__table__.name)
 
     def fund_nav_from_rq(self, start_date, end_date):
         df = RawDataApi().get_rq_fund_nav(start_date, end_date)
         df['fund_id'] = df.apply(
-            lambda x: self._get_fund_id_from_order_book_id(x['order_book_id'], x['datetime']), axis=1)
+            lambda x: self._data_helper._get_fund_id_from_order_book_id(x['order_book_id'], x['datetime']), axis=1)
         df['subscribe_status'] = df['subscribe_status'].map(self._status_mapping)
         df['redeem_status'] = df['redeem_status'].map(self._status_mapping)
         df = df[df['fund_id'].notna()]
-        self._upload_basic(df, FundNav.__table__.name)
+        self._data_helper._upload_basic(df, FundNav.__table__.name)
 
     def fund_nav(self, start_date, end_date):
         try:
-            start_date = datetime.datetime.strptime(start_date, '%Y%m%d') - datetime.timedelta(days=3)
+            start_date = datetime.datetime.strptime(start_date, '%Y%m%d') - datetime.timedelta(days=5)
+            del_date = start_date + datetime.timedelta(days=1)
             start_date = start_date.strftime('%Y%m%d')
             df = RawDataApi().get_em_fund_nav(start_date, end_date)
             df['fund_id'] = df.apply(
-                lambda x: self._get_fund_id_from_order_book_id(x['CODES'].split('.')[0], x['DATES']), axis=1)
+                lambda x: self._data_helper._get_fund_id_from_order_book_id(x['CODES'].split('.')[0], x['DATES']), axis=1)
             df = df.drop(['CODES'], axis=1)
             df = df[df['fund_id'].notna()]
             df = df.rename(columns={
@@ -143,8 +121,9 @@ class BasicDataPart1(object):
             df['change_rate'] = df.adjusted_net_value / df.adjusted_net_value.shift(1) - 1
             df['weekly_yield'] = df['weekly_yield'].map(lambda x: x / 100.0)
             end_date = datetime.datetime.strptime(end_date, '%Y%m%d').date()
-            df = df[df.datetime == end_date]
-            self._upload_basic(df, FundNav.__table__.name)
+            BasicDataApi().delete_fund_nav(del_date, end_date)
+            df = df[df.datetime >= del_date.date()]
+            self._data_helper._upload_basic(df, FundNav.__table__.name)
             return True
         except Exception as e:
             print(e)
@@ -200,8 +179,8 @@ class BasicDataPart1(object):
             df = pd.concat(df_list).drop(['order_book_id'], axis=1)
             df['datetime'] = pd.to_datetime(df['datetime'])
             end_date = datetime.datetime.strptime(end_date, '%Y%m%d').date()
-            df = df[df.datetime==end_date]
-            self._upload_basic(df, IndexPrice.__table__.name)
+            df = df[df.datetime == end_date]
+            self._data_helper._upload_basic(df, IndexPrice.__table__.name)
             return True
         except Exception as e:
             print(e)
@@ -236,7 +215,7 @@ class BasicDataPart1(object):
             df = pd.concat(res, axis=1, sort=False)
             df['fund_id'] = df.index
             df['update_time'] = datetime.date.today()
-            self._upload_basic(df, FundRatingLatest.__table__.name, to_truncate=True)
+            self._data_helper._upload_basic(df, FundRatingLatest.__table__.name, to_truncate=True)
             return True
         except Exception as e:
             print(e)
@@ -249,13 +228,13 @@ class BasicDataPart1(object):
             stock_post_price_df = RawDataApi().get_rq_stock_post_price(start_date, end_date)
 
             stock_post_price_df['adj_close'] = stock_post_price_df['close']
-            stock_post_price_df = stock_post_price_df.filter(items=['adj_close', 'datetime', 'order_book_id'], 
+            stock_post_price_df = stock_post_price_df.filter(items=['adj_close', 'datetime', 'order_book_id'],
                 axis='columns')
 
             stock_price_merge_df = stock_price_df.merge(stock_post_price_df, how='left', on=['datetime', 'order_book_id'])
             stock_price_merge_df['post_adj_factor'] = stock_price_merge_df['adj_close'] / stock_price_merge_df['close']
             stock_price_merge_df = stock_price_merge_df.rename(columns={'order_book_id': 'stock_id'})
-            self._upload_basic(stock_price_merge_df, StockPrice.__table__.name)
+            self._data_helper._upload_basic(stock_price_merge_df, StockPrice.__table__.name)
             return True
         except Exception as e:
             print(e)
@@ -281,11 +260,11 @@ class BasicDataPart1(object):
 
             df = RawDataApi().get_rq_fund_indicator(start_date, end_date)
             df['fund_id'] = df.apply(
-                lambda x: self._get_fund_id_from_order_book_id(x['order_book_id'], x['datetime']), axis=1)
+                lambda x: self._data_helper._get_fund_id_from_order_book_id(x['order_book_id'], x['datetime']), axis=1)
             df = df.rename(columns=column_rename_dict).drop(columns=column_drop_list)
             df = df[df['fund_id'].notnull()]
 
-            self._upload_basic(df, FundRet.__table__.name)
+            self._data_helper._upload_basic(df, FundRet.__table__.name)
             return True
         except Exception as e:
             print(e)
@@ -295,7 +274,7 @@ class BasicDataPart1(object):
     def stock_info(self):
         try:
             df = RawDataApi().get_stock_info()
-            self._upload_basic(df, StockInfo.__table__.name, to_truncate=True)
+            self._data_helper._upload_basic(df, StockInfo.__table__.name, to_truncate=True)
             return True
         except Exception as e:
             print(e)
@@ -307,10 +286,10 @@ class BasicDataPart1(object):
             # Since fund_size in basic db is latest snapshot, we only use end_date as param
             df = RawDataApi().get_em_fund_scale(end_date, end_date)
             df['fund_id'] = df.apply(
-                lambda x: self._get_fund_id_from_order_book_id(x['CODES'].split('.')[0], x['DATES']), axis=1)
+                lambda x: self._data_helper._get_fund_id_from_order_book_id(x['CODES'].split('.')[0], x['DATES']), axis=1)
             df = df.drop(['CODES', 'DATES'], axis = 1).rename(columns={'FUNDSCALE': 'latest_size'})
             df = df[df['fund_id'].notnull()]
-            self._upload_basic(df, FundSize.__table__.name, to_truncate=True)
+            self._data_helper._upload_basic(df, FundSize.__table__.name, to_truncate=True)
             return True
         except Exception as e:
             print(e)
@@ -321,13 +300,13 @@ class BasicDataPart1(object):
         try:
             df = RawDataApi().get_em_tradedates(start_date, end_date)
             df = df.rename(columns={'TRADEDATES': 'datetime'})
-            self._upload_basic(df, TradingDayList.__table__.name)
+            self._data_helper._upload_basic(df, TradingDayList.__table__.name)
             return True
         except Exception as e:
             print(e)
             traceback.print_stack()
             return False
-    
+
     def em_index_price(self, start_date, end_date):
         try:
             start_date = datetime.datetime.strptime(start_date, '%Y%m%d') - datetime.timedelta(days=5)
@@ -355,13 +334,43 @@ class BasicDataPart1(object):
             BasicDataApi().delete_index_price(index_id_list, del_date, end_date)
             df = pd.concat(res, axis=0, sort=False)
             df = df[df.datetime >= del_date.date()]
-            self._upload_basic(df, IndexPrice.__table__.name)
+            self._data_helper._upload_basic(df, IndexPrice.__table__.name)
             return True
         except Exception as e:
             print(e)
             traceback.print_stack()
             return False
 
+    def fund_size_hold_rate_history(self):
+        start_date = '20100101'
+        end_date = '20200612'
+        raw = RawDataApi()
+        basic = BasicDataApi()
+        fund_holding_rate = raw.get_em_fund_holding_rate(start_date=start_date)
+        fund_scale = raw.get_em_fund_scale(start_date=start_date, end_date=end_date)
+        fund_holding_rate['fund_id'] = fund_holding_rate.apply(
+            lambda x: self._data_helper._get_fund_id_from_order_book_id(x['CODES'].split('.')[0], x['DATES']), axis=1)
+        fund_scale['fund_id'] = fund_scale.apply(
+            lambda x: self._data_helper._get_fund_id_from_order_book_id(x['CODES'].split('.')[0], x['DATES']), axis=1)
+        fund_holding_rate = fund_holding_rate.drop(['CODES', '_update_time'], axis=1).rename(columns={'DATES':'datetime','HOLDPERSONALHOLDINGPCT':'personal_holds','HOLDINSTIHOLDINGPCT':'institution_holds'})
+        fund_scale = fund_scale.drop(['CODES','_update_time'], axis=1).rename(columns={'DATES':'datetime','FUNDSCALE':'size'})
+        fund_scale = fund_scale.dropna().drop_duplicates(subset=['size','fund_id'],keep='first')
+        fund_holding_rate = fund_holding_rate.dropna()
+        fund_list = list(set(fund_scale.fund_id))
+        res = []
+        for fund_id in fund_list:
+            fund_hold_i = fund_holding_rate[fund_holding_rate.fund_id == fund_id].drop('fund_id',axis=1).set_index('datetime')
+            fund_scale_i = fund_scale[fund_scale.fund_id == fund_id].drop('fund_id',axis=1).set_index('datetime')
+            _df = fund_scale_i.join(fund_hold_i).fillna(method='ffill').reset_index()
+            _df['fund_id'] = fund_id
+            res.append(_df)
+        df = pd.concat(res)
+        df = df.drop_duplicates(subset=['datetime','fund_id'],keep='first')
+        df = df.fillna(0)
+        con_list = (df.personal_holds + df.institution_holds) == 100
+        df = df[con_list]
+        self._data_helper._upload_basic(df, Fund_size_and_hold_rate.__table__.name)
+
     def process_all(self, start_date, end_date):
         failed_tasks = []
 
@@ -384,7 +393,7 @@ class BasicDataPart1(object):
         # TODO: Depend on RQ, change to EM
         if not self.fund_rating_latest():
             failed_tasks.append('fund_rating_latest')
-            
+
         if not self.fund_size(start_date, end_date):
             failed_tasks.append('fund_size')
 
@@ -399,6 +408,7 @@ class BasicDataPart1(object):
 
         return failed_tasks
 
+
 if __name__ == '__main__':
     bdp = BasicDataPart1()
     # start_date = '20200428'
