@@ -9,8 +9,8 @@ import datetime
 
 class FundScoreProcessor(object):
 
-    def __init__(self):
-        self.updated_count = {}
+    def __init__(self, data_helper):
+        self._data_helper = data_helper
 
     def init(self, start_time, end_time):
         self._dm = FundDataManager(start_time=start_time, end_time=end_time, score_manager=FundScoreManager())
@@ -25,28 +25,23 @@ class FundScoreProcessor(object):
         self.date_list = sorted(list(set(self.date_list)))
         self.desc_dic = self._dm.dts.fund_info[['fund_id','desc_name']].set_index('fund_id').to_dict('index')
 
-    def upload_derived(self, df, table_name):
-        print(table_name)
-        df.to_sql(table_name, DerivedDatabaseConnector().get_engine(), index=False, if_exists='append')
-
-        if table_name not in self.updated_count:
-            self.updated_count[table_name] = 0
-        self.updated_count[table_name] += df.shape[0]
-
     def calculate_score(self):
         res = []
         for dt in self.date_list:
             for index_id in self.index_list:
-                res_i, _ = self._dm.get_fund_score(dt, index_id)
-                if res_i:
-                    for fund_id, score_i in res_i.items():
-                        dic = {
-                            'fund_id':fund_id,
-                            'score':score_i,
-                            'datetime':dt,
-                            'index_id':index_id
-                        }
-                        res.append(dic)
+                try:
+                    res_i, _ = self._dm.get_fund_score(dt, index_id)
+                    if res_i:
+                        for fund_id, score_i in res_i.items():
+                            dic = {
+                                'fund_id':fund_id,
+                                'score':score_i,
+                                'datetime':dt,
+                                'index_id':index_id
+                            }
+                            res.append(dic)
+                except:
+                    continue
         score_df = pd.DataFrame(res)
         score_df['tag_name'] = score_df['index_id'].map(lambda x: self.index_tag.get(x))
         score_df.drop(['index_id'], axis=1, inplace=True)
@@ -55,7 +50,7 @@ class FundScoreProcessor(object):
         score_df['tag_method'] = 'm0'
         score_df['score_method'] = 9
         score_df['desc_name'] = score_df['fund_id'].map(lambda x: self.desc_dic[x]['desc_name'])
-        self.upload_derived(score_df, FundScore.__table__.name)         
+        self._data_helper._upload_derived(score_df, FundScore.__table__.name)
 
     def process(self, start_date, end_date):
         failed_tasks = []
