@@ -17,10 +17,12 @@ from ...wrapper.mysql import RawDatabaseConnector
 from ...view.raw_models import *
 from ...api.basic import BasicDataApi
 from ...api.raw import RawDataApi
+from .raw_data_helper import RawDataHelper
 
 class EmRawDataDownloader(object):
 
-    def __init__(self):
+    def __init__(self, data_helper):
+        self._data_helper = data_helper
         # ForceLogin
         # 取值0，当线上已存在该账户时，不强制登录
         # 取值1，当线上已存在该账户时，强制登录，将前一位在线用户踢下线
@@ -32,16 +34,6 @@ class EmRawDataDownloader(object):
 
         self._scale_change_dates = ['0331', '0630', '0930', '1231']
 
-        self.updated_count = {}
-
-    def _upload_raw(self, df: pd.DataFrame, table_name: str):
-        print(table_name)
-        df.to_sql(table_name, RawDatabaseConnector().get_engine(), index=False, if_exists='append')
-
-        if table_name not in self.updated_count:
-            self.updated_count[table_name] = 0
-        self.updated_count[table_name] += df.shape[0]
-
     def _main_callback(self, quantdata):
         '''
         mainCallback 是主回调函数，可捕捉连接错误
@@ -82,7 +74,7 @@ class EmRawDataDownloader(object):
                     fund_nav = fund_nav[(fund_nav['DATES'] != '--')
                                         & (~fund_nav['DATES'].isnull())]
                     fund_nav = fund_nav.replace('--', np.nan)
-                    self._upload_raw(fund_nav, EmFundNav.__table__.name)
+                    self._data_helper._upload_raw(fund_nav, EmFundNav.__table__.name)
                 else:
                     continue
             return True
@@ -102,7 +94,7 @@ class EmRawDataDownloader(object):
             'CLOSE': 'close'
         })
 
-        self._upload_raw(df, EmIndexPrice.__table__.name)
+        self._data_helper._upload_raw(df, EmIndexPrice.__table__.name)
 
     def em_fund_scale_history(self):
         date_funds = {}
@@ -139,15 +131,35 @@ class EmRawDataDownloader(object):
                                f"EndDate={d},Ispandas=1").reset_index()
                     df['DATES'] = d
 
-                    self._upload_raw(df, EmFundScale.__table__.name)
+                    self._data_helper._upload_raw(df, EmFundScale.__table__.name)
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))
                 return None
 
+    def em_fund_holding_rate_history(self):
+        # 机构持有比例半年更新，到期日未必有数据，会有延迟， 含已终止基金
+        fund_id_result = c.sector('507010', '2020-06-10')
+        fund_list = fund_id_result.Data[0::2]
+        fund_id_str = ','.join(fund_list)
+        y_list = [str(y) for y in range(2010, 2020,1)]
+        d_list = []
+        for y in y_list:
+            for d in ['-06-30', '-12-31']:
+                d_list.append(y+d)
+        for d in d_list:
+            data=c.css(fund_id_str,"HOLDPERSONALHOLDINGPCT,HOLDINSTIHOLDINGPCT",f"ReportDate={d} ,Ispandas=1")
+            data['DATES'] = d
+            data = data.reset_index().dropna()
+            self._data_helper._upload_raw(data, EmFundHoldingRate.__table__.name)
+            print(f'finish {d} ')
+
     def em_fund_nav(self, start_date, end_date):
         try:
             # Get all fund ids, 功能函数-板块成分
             # http://quantapi.eastmoney.com/Cmd/Sector?from=web
+            start_date = datetime.datetime.strptime(start_date, '%Y%m%d') - datetime.timedelta(days=5)
+            start_date = start_date.strftime('%Y%m%d')
+            self.delete_em_fund_nav(start_date, end_date)
             fund_id_result = c.sector('507013', end_date)
             if fund_id_result.ErrorCode != 0:
                 print(f'Failed to get fund id list: {fund_id_result.ErrorMsg}')
@@ -163,13 +175,22 @@ class EmRawDataDownloader(object):
             df.reset_index(inplace=True)
             df['UNITYIELD10K'] = df['10KUNITYIELD']
             df = df.drop(['10KUNITYIELD'], axis=1)
-            self._upload_raw(df, EmFundNav.__table__.name)
+            self._data_helper._upload_raw(df, EmFundNav.__table__.name)
             return True
         except Exception as e:
             print(e)
             traceback.print_stack()
             return False
 
+    def delete_em_fund_nav(self, start_date, end_date):
+        try:
+            RawDataApi().delete_em_fund_nav(start_date, end_date)
+            return True
+        except Exception as e:
+            print(e)
+            traceback.print_stack()
+            return False    
+
     def em_fund_scale(self, start_date, end_date):
         try:
             # Get all fund ids, 功能函数-板块成分
@@ -185,7 +206,7 @@ class EmRawDataDownloader(object):
             for d in self._get_date_list(start_date, end_date):
                 df = c.css(fund_id_str, 'FUNDSCALE', f'EndDate={d},Ispandas=1').reset_index()
                 df['DATES'] = d
-                self._upload_raw(df, EmFundScale.__table__.name)
+                self._data_helper._upload_raw(df, EmFundScale.__table__.name)
             return True
         except Exception as e:
             print(e)
@@ -200,11 +221,11 @@ class EmRawDataDownloader(object):
             res = []
             if isinstance(em_ids, list):
                 for em_id in em_ids:
-                    df = c.csd(em_id, "CLOSE", start_date, end_date,"period=1,adjustflag=1,curtype=1,order=1,market=CNSESH,Ispandas=1")
+                    df = c.csd(em_id, "CLOSE", start_date, end_date, "period=1,adjustflag=1,curtype=1,order=1,market=CNSESH,Ispandas=1")
                     res.append(df)
                 df = pd.concat(res)
             else:
-                df = c.csd(em_ids, "CLOSE", start_date, end_date,"period=1,adjustflag=1,curtype=1,order=1,market=CNSESH,Ispandas=1")
+                df = c.csd(em_ids, "CLOSE", start_date, end_date, "period=1,adjustflag=1,curtype=1,order=1,market=CNSESH,Ispandas=1")
 
             df = df.reset_index().rename(columns={
                 'CODES': 'em_id',
@@ -212,7 +233,7 @@ class EmRawDataDownloader(object):
                 'CLOSE': 'close'
             })
 
-            self._upload_raw(df, EmIndexPrice.__table__.name)
+            self._data_helper._upload_raw(df, EmIndexPrice.__table__.name)
             return True
         except Exception as e:
             print(e)
@@ -256,11 +277,10 @@ class EmRawDataDownloader(object):
         if df is not None:
             df.reset_index(inplace=True)
             print(df)
-            # self._upload_raw(df, EmIndexPrice.__table__.name)
+            # self._data_helper._upload_raw(df, EmIndexPrice.__table__.name)
 
     # 从em客户端导出数据后读取并处理
     def em_stock_price_history(self, stock_price_file_dir: str):
-        import os
         for index, filename in enumerate(os.listdir(stock_price_file_dir)):
             if not filename.endswith(".xls"):
                 continue
@@ -289,7 +309,7 @@ class EmRawDataDownloader(object):
             # 过滤掉个股未上市时的数据，以及日期为nan的行（表示整行为无效数据）
             stock_price = stock_price.loc[stock_price['TRADESTATUS'] != '未上市'].loc[~stock_price['DATES'].isna()].rename(columns=lambda x: EmStockPrice.__getattribute__(EmStockPrice, x).name)
             # 更新到db
-            self._upload_raw(stock_price, EmStockPrice.__table__.name)
+            self._data_helper._upload_raw(stock_price, EmStockPrice.__table__.name)
 
     def em_stock_price(self, start_date: str, end_date: str):
         try:
@@ -324,13 +344,41 @@ class EmRawDataDownloader(object):
 
             df = pd.concat(df_list).reset_index().rename(columns=lambda x: EmStockPrice.__getattribute__(EmStockPrice, x).name)
             # 更新到db
-            self._upload_raw(df, EmStockPrice.__table__.name)
+            self._data_helper._upload_raw(df, EmStockPrice.__table__.name)
             return True
         except Exception as e:
             print(e)
             traceback.print_stack()
             return False
 
+    # 从em客户端导出数据后读取并处理
+    def em_stock_post_price_history(self, stock_post_price_file_dir: str):
+        for index, filename in enumerate(os.listdir(stock_post_price_file_dir)):
+            if not filename.endswith(".xls"):
+                continue
+            index += 1
+            print(f'{index}: {filename}')
+            stock_price = pd.concat(pd.read_excel(os.path.join(stock_post_price_file_dir, filename), sheet_name=None, na_values='--'),
+                                    ignore_index=True)
+            # 将列名字改为从API获取到数据时的列名字，便于后边统一处理
+            stock_price = stock_price.drop(['简称'], axis=1).rename(columns={
+                '代码': 'CODES',
+                '时间': 'DATES',
+                '开盘价(元)': 'OPEN',
+                '收盘价(元)': 'CLOSE',
+                '最高价(元)': 'HIGH',
+                '最低价(元)': 'LOW',
+                '前收盘价(元)': 'PRECLOSE',
+                '均价(元)': 'AVERAGE',
+                '成交量(股)': 'VOLUME',
+                '成交金额(元)': 'AMOUNT',
+                '交易状态': 'TRADESTATUS',
+            })
+            # 过滤掉个股未上市时的数据，以及日期为nan的行（表示整行为无效数据）
+            stock_price = stock_price.loc[stock_price['TRADESTATUS'] != '未上市'].loc[~stock_price['DATES'].isna()].rename(columns=lambda x: EmStockPostPrice.__getattribute__(EmStockPostPrice, x).name)
+            # 更新到db
+            self._data_helper._upload_raw(stock_price, EmStockPostPrice.__table__.name)
+
     def em_stock_post_price(self, start_date: str, end_date: str):
         try:
             # 获取区间内所有交易日
@@ -364,7 +412,7 @@ class EmRawDataDownloader(object):
 
             df = pd.concat(df_list).reset_index().rename(columns=lambda x: EmStockPostPrice.__getattribute__(EmStockPostPrice, x).name)
             # 更新到db
-            self._upload_raw(df, EmStockPostPrice.__table__.name)
+            self._data_helper._upload_raw(df, EmStockPostPrice.__table__.name)
             return True
         except Exception as e:
             print(e)
@@ -420,7 +468,7 @@ class EmRawDataDownloader(object):
             # 不需要存日期的列，去除索引并做列名转换
             df = df.drop(columns='DATES').reset_index().rename(columns=lambda x: EmStockInfo.__getattribute__(EmStockInfo, x).name)
             # 更新到db
-            self._upload_raw(df, EmStockInfo.__table__.name)
+            self._data_helper._upload_raw(df, EmStockInfo.__table__.name)
 
     def add_columns_to_fin_fac(self, end_date: str):
         # 获取所有A股股票的ID, 功能函数-板块成分
@@ -432,20 +480,23 @@ class EmRawDataDownloader(object):
             return
 
         # 获取主营收入构成(按产品)
-        df = c.css(stock_id_result.Codes, "MBSALESCONS", f'ReportDate={end_date},type=2,Ispandas=1')
+        df = c.css(stock_id_result.Codes, "BALANCESTATEMENT_195", f'ReportDate={end_date},type=1,Ispandas=1')
         if not isinstance(df, pd.DataFrame) and df.ErrorCode != 0:
             print(f'failed to get stock financial factors: {df.ErrorMsg}')
             return
 
+        count = 0
         Session = sessionmaker(RawDatabaseConnector().get_engine())
         db_session = Session()
         for row in db_session.query(EmStockFinFac):
-            if row.DATES != datetime.datetime.strptime(end_date, '%Y%m%d').date().year:
+            if row.DATES != datetime.datetime.strptime(end_date, '%Y%m%d').date():
                 continue
             if row.CODES in df.index:
-                row.MBSALESCONS_P = df.at[row.CODES, 'MBSALESCONS']
+                row.BALANCESTATEMENT_195 = df.at[row.CODES, 'BALANCESTATEMENT_195']
+                count += 1
         db_session.commit()
         db_session.close()
+        print(f'append {count} value(s) to fin fac table')
 
     def em_stock_fin_fac(self, end_date: str):
         # 获取所有A股股票的ID, 功能函数-板块成分
@@ -461,8 +512,8 @@ class EmRawDataDownloader(object):
         df_no_param = c.css(stock_id_result.Codes,
                             'EBIT,EBITDA,EXTRAORDINARY,LOWERDIANDNI,GROSSMARGIN,OPERATEINCOME,INVESTINCOME,EBITDRIVE,TOTALCAPITAL,WORKINGCAPITAL,\
                              NETWORKINGCAPITAL,TANGIBLEASSET,RETAINED,INTERESTLIBILITY,NETLIBILITY,EXINTERESTCL,EXINTERESTNCL,FCFF,FCFE,DA,FCFFDRIVE,\
-                             PERFORMANCEEXPRESSCUTPARENTNI,PERFORMANCEEXPRESSPARENTNI,MBSALESCONS,GPMARGIN,NPMARGIN,EXPENSETOOR,INVTURNRATIO,ARTURNRATIO,ROEAVG,ROEWA,EPSBASIC,EPSDILUTED,\
-                             BPS,BALANCESTATEMENT_25,BALANCESTATEMENT_46,BALANCESTATEMENT_93,BALANCESTATEMENT_103,BALANCESTATEMENT_141,\
+                             PERFORMANCEEXPRESSPARENTNI,MBSALESCONS,GPMARGIN,NPMARGIN,EXPENSETOOR,INVTURNRATIO,ARTURNRATIO,ROEAVG,ROEWA,EPSBASIC,EPSDILUTED,\
+                             BPS,BALANCESTATEMENT_25,BALANCESTATEMENT_46,BALANCESTATEMENT_93,BALANCESTATEMENT_103,BALANCESTATEMENT_141,BALANCESTATEMENT_195,\
                              BALANCESTATEMENT_140,INCOMESTATEMENT_9,INCOMESTATEMENT_48,INCOMESTATEMENT_60,INCOMESTATEMENT_61,INCOMESTATEMENT_85,INCOMESTATEMENT_127,\
                              INCOMESTATEMENT_14,CASHFLOWSTATEMENT_39,CASHFLOWSTATEMENT_59,CASHFLOWSTATEMENT_77,CASHFLOWSTATEMENT_82,CASHFLOWSTATEMENT_86',
                             f'ReportDate={end_date},type=1,Ispandas=1')
@@ -509,7 +560,28 @@ class EmRawDataDownloader(object):
             df.loc[:, 'DATES'] = end_date
             df = df.reset_index().rename(columns=lambda x: EmStockFinFac.__getattribute__(EmStockFinFac, x).name)
             # 更新到db
-            self._upload_raw(df, EmStockFinFac.__table__.name)
+            self._data_helper._upload_raw(df, EmStockFinFac.__table__.name)
+
+    # 从em客户端导出数据后读取并处理
+    def em_stock_daily_info_history(self, stock_daily_info_file_dir: str):
+        for index, filename in enumerate(os.listdir(stock_daily_info_file_dir)):
+            if not filename.endswith(".xls"):
+                continue
+            index += 1
+            print(f'{index}: {filename}')
+            stock_daily_info = pd.concat(pd.read_excel(os.path.join(stock_daily_info_file_dir, filename), sheet_name=None, na_values='--'),
+                                         ignore_index=True)
+            # 将列名字改为从API获取到数据时的列名字，便于后边统一处理
+            stock_daily_info = stock_daily_info.drop(['简称'], axis=1).rename(columns={
+                '代码': 'CODES',
+                '时间': 'DATES',
+                '总股本(股)': 'TOTALSHARE',
+            })
+
+            stock_daily_info = stock_daily_info.loc[~stock_daily_info['DATES'].isna()].rename(columns=lambda x: EmStockDailyInfo.__getattribute__(EmStockDailyInfo, x).name)
+
+            # 更新到db
+            self._data_helper._upload_raw(stock_daily_info, EmStockDailyInfo.__table__.name)
 
     def em_stock_daily_info(self, start_date: str, end_date: str, predict_year: int = 0):
         try:
@@ -547,7 +619,7 @@ class EmRawDataDownloader(object):
 
             df = pd.concat(df_list).reset_index().rename(columns=lambda x: EmStockDailyInfo.__getattribute__(EmStockDailyInfo, x).name)
             # 更新到db
-            self._upload_raw(df, EmStockDailyInfo.__table__.name)
+            self._data_helper._upload_raw(df, EmStockDailyInfo.__table__.name)
             return True
         except Exception as e:
             print(e)
@@ -570,7 +642,7 @@ class EmRawDataDownloader(object):
                 infos.append(v)
             df = pd.DataFrame(infos, columns=['SECURITYCODE', 'APPROVENOTICEDATE', 'PLANNOTICEDDATE',
                                               'NEWPROGRESS', 'SUMFINA_T', 'ATTACHNAME', 'ADDPURPOSE'])
-            self._upload_raw(df.rename(columns=lambda x: EmStockRefinancing.__getattribute__(EmStockRefinancing, x).name), EmStockRefinancing.__table__.name)
+            self._data_helper._upload_raw(df.rename(columns=lambda x: EmStockRefinancing.__getattribute__(EmStockRefinancing, x).name), EmStockRefinancing.__table__.name)
             return True
         except Exception as e:
             print(e)
@@ -627,7 +699,7 @@ class EmRawDataDownloader(object):
 
             # 更新到db
             df = pd.concat(df_list).reset_index().rename(columns=lambda x: EmIndexVal.__getattribute__(EmIndexVal, x).name)
-            self._upload_raw(df, EmIndexVal.__table__.name)
+            self._data_helper._upload_raw(df, EmIndexVal.__table__.name)
             return True
         except Exception as e:
             print(e)
@@ -643,13 +715,39 @@ class EmRawDataDownloader(object):
 
             tradedates = tradedates_result.Data
             df = pd.DataFrame(tradedates, columns=['TRADEDATES'])
-            self._upload_raw(df, EmTradeDates.__table__.name)
+            self._data_helper._upload_raw(df, EmTradeDates.__table__.name)
             return True
         except Exception as e:
             print(e)
             traceback.print_stack()
             return False
 
+    def em_index_component(self, start_date: str, end_date: str):
+        try:
+            index_info = BasicDataApi().get_index_info()
+
+            date_list = self._get_date_list(start_date, end_date)
+            s_list: List[pd.Series] = []
+            for row in index_info[~index_info['em_plate_id'].isna()].loc[:, ['index_id', 'em_id', 'em_plate_id']].itertuples(index=False):
+                for date in date_list:
+                    stock_id_result = c.sector(row.em_plate_id, date)
+                    if stock_id_result.ErrorCode != 0:
+                        # print(f'failed to get stock id list: {stock_id_result.ErrorMsg}')
+                        continue
+
+                    stock_id_list: List[str] = []
+                    for i in range(0, len(stock_id_result.Data), 2):
+                        stock_id_list.append(stock_id_result.Data[i])
+                    if len(stock_id_list) != 0:
+                        s_list.append(pd.Series(row._asdict()).append(pd.Series({'datetime': date, 'stock_list': ','.join(stock_id_list)})))
+
+            # 更新到db
+            self._data_helper._upload_raw(pd.DataFrame(s_list), EmIndexComponent.__table__.name)
+            return True
+        except Exception as e:
+            print(e)
+            traceback.print_stack()
+            return
 
     def download_all(self, start_date, end_date):
         failed_tasks = []
@@ -676,24 +774,32 @@ class EmRawDataDownloader(object):
 
         if not self.em_index_val(start_date, end_date):
             failed_tasks.append('em_index_val')
-        
+
         if not self.em_stock_daily_info(start_date, end_date):
             failed_tasks.append('em_stock_daily_info')
 
+        if not self.em_index_component(start_date, end_date):
+            failed_tasks.append('em_index_component')
+
         return failed_tasks
 
 
 if __name__ == '__main__':
-    em = EmRawDataDownloader()
+    data_helper = RawDataHelper()
+    em = EmRawDataDownloader(data_helper)
     # em.em_index_val('20200428', '20200428')
     # em.em_fund_nav('20200415', '20200427')
     # em.em_index_price_history('H00140.SH', '20050101', '20200419')
     # em.em_fund_scale_history()
     # em.em_stock_fin_fac('20161231')
     # em.em_stock_info('20200424')
-    # em.em_stock_price('20200428', '20200428')
-    # em.em_stock_daily_info('20200228', '20200228')
-    # em.em_stock_price_history('./surfing/data/fund/raw/stock_price')
+    # em.em_stock_price('20041231', '20041231')
+    # em.em_stock_post_price('20041231', '20041231')
+    # em.em_stock_daily_info('20041231', '20041231')
+    # em.em_stock_price_history('./')
+    # em.em_stock_post_price_history('./')
+    # em.em_stock_daily_info_history('./')
     # em.em_stock_refinancing('20170101', '20181231')
     # em.add_columns_to_fin_fac('20190930')
-    em.em_fund_scale('20200513', '20200513')
+    # em.em_fund_scale('20200513', '20200513')
+    # em.em_index_component('20200528', '20200529')
