@@ -33,6 +33,7 @@ class IndexDailyCollectionProcessor(object):
             except Exception as e:
                 print('Failed get_index_volatility <err_msg> {}'.format(e))
 
+    # TODO: Use basic.index_valuation_develop table
     def get_index_valuation(self):
         with DerivedDatabaseConnector().managed_session() as mn_session:
             try:
@@ -117,8 +118,7 @@ class IndexDailyCollectionProcessor(object):
                     IndexPrice.close,
                     IndexPrice.high,
                     IndexPrice.open,
-                    IndexPrice.total_turnover,
-                    IndexPrice.ret,
+                    IndexPrice.total_turnover
                 ).filter(IndexPrice.datetime==latest_time)
                 df = pd.read_sql(query.statement, query.session.bind)
                 df = df.rename(columns={'datetime': 'price_datetime'})
@@ -130,6 +130,12 @@ class IndexDailyCollectionProcessor(object):
 
     def append_data(self, table_name, data_append_directly_data_df):
         if not data_append_directly_data_df.empty:
+            with ViewDatabaseConnector().managed_session() as mn_session:
+                try:
+                    mn_session.execute(f'TRUNCATE TABLE {table_name}')
+                    mn_session.commit()
+                except Exception as e:
+                    print(f'Failed to truncate table {table_name} <err_msg> {e}')
             data_append_directly_data_df.to_sql(table_name, ViewDatabaseConnector().get_engine(), index = False, if_exists = 'append')
             print('新数据已插入')
         else:
