@@ -6,6 +6,10 @@ class ViewDataProcessor(object):
         self.fund_daily_collection_processor = FundDailyCollectionProcessor()
         self.index_daily_collection_processor = IndexDailyCollectionProcessor()
 
-    def process(self, start_date, end_date):
-        self.fund_daily_collection_processor.compute_daily_collection()
-        self.index_daily_collection_processor.collection_daily_index()
+    def process_all(self, start_date, end_date):
+        failed_tasks = []
+
+        failed_tasks.extend(self.fund_daily_collection_processor.process())
+        failed_tasks.extend(self.index_daily_collection_processor.process())
+
+        return failed_tasks
