@@ -15,6 +15,8 @@ class FundDataTables:
     fund_nav: pd.DataFrame = None
     fund_indicator: pd.DataFrame = None
     fund_unit_nav: pd.DataFrame = None
+    fund_size: pd.DataFrame = None
+    fund_com_hold: pd.DataFrame = None
     # fund helps
     fund_list: set = None
     index_list: set = None
@@ -37,12 +39,11 @@ class FundDataTables:
         self.index_fund_indicator_pack = self.fund_indicator.pivot_table(index=['index_id', 'datetime', 'fund_id'])
         self.index_fund_indicator_pack['ir'] = self.index_fund_indicator_pack['alpha'] / self.index_fund_indicator_pack['track_err']
         self.index_list = set(self.index_fund_indicator_pack.index.levels[0])
-        
+
     def fix_data(self):
         # 将 fix_data_v1 去掉的原因是目前 basic 改用choice数据源，nav数据没有这个问题了。
         # self.fix_data_v1()
         self._agg_indicator()
-        pass
 
     def fix_data_v1(self):
         '''
