@@ -15,6 +15,8 @@ from ..view.derived_models import *
 
 class FundDataManager(DataManager):
 
+    BLACK_SHEEP = ['020035!0', '020036!0'] #摘牌基金 发生大额申赎，导致净值剧烈波动， 用东财机构持有权重数据无法过滤
+
     def __init__(self, start_time=None, end_time=None, fund_score_param: FundScoreParam=None, score_manager: FundScoreManager=None):
         DataManager.__init__(self, 
             start_time or datetime.datetime(2005, 1, 1), 
@@ -44,15 +46,20 @@ class FundDataManager(DataManager):
             self.dts.fund_info = fetch_table(FundInfo)
             _tm_basic_fetch_info = time.time()
             no_mmf_index_list = list(set(index_list) - set(['mmf']))
+            self.dts.fund_info.loc[self.dts.fund_info.national_debt_extension == 1, 'index_id'] = 'national_debt'  
             _filter_fund_info = self.dts.fund_info[
-                    self.dts.fund_info.index_id.isin(no_mmf_index_list)
-                    & ((self.dts.fund_info.structure_type == 0) | (self.dts.fund_info.structure_type == 1)) 
-                    & (self.dts.fund_info.is_etf != 1) 
-                    & (self.dts.fund_info.currency == 'CNY')
-                    & (self.dts.fund_info.is_open == 1)
-                    | (self.dts.fund_info.is_selected_mmf == 1)
+                    (
+                        self.dts.fund_info.index_id.isin(no_mmf_index_list)
+                        & ((self.dts.fund_info.structure_type == 0) | (self.dts.fund_info.structure_type == 1)) 
+                        & (self.dts.fund_info.is_etf != 1) 
+                        & (self.dts.fund_info.currency == 'CNY')
+                        & (self.dts.fund_info.is_open == 1)
+                        | (self.dts.fund_info.is_selected_mmf == 1)
+                    )
+                    & (~self.dts.fund_info.fund_id.isin(self.BLACK_SHEEP))
                 ]
             _tm_basic_filter_info = time.time()
+            
             self.dts.fund_list = set(_filter_fund_info.fund_id)
             self.dts.fund_index_map = {cur.fund_id: cur.index_id for idx, cur in _filter_fund_info.iterrows()}
             _tm_basic_prep_struct = time.time()
@@ -81,6 +88,26 @@ class FundDataManager(DataManager):
             self.dts.fund_nav = _nav_df.pivot_table(index='datetime', columns='fund_id', values='adjusted_net_value').fillna(method='ffill')
             self.dts.fund_unit_nav = _nav_df.pivot_table(index='datetime', columns='fund_id', values='unit_net_value').fillna(method='ffill')
             _tm_basic_fetch_fund_nav = time.time()
+            #fund size and fund com hold
+            # 机构持仓比例每六个月更新，提前取半年以上, 填充空白值用
+            # fund_com_hold fund_size 和 fund_nav同结构，日期纵轴，基金代码横轴
+            _fund_size_and_hold_rate_query = quant_session.query(
+                    Fund_size_and_hold_rate.fund_id,
+                    Fund_size_and_hold_rate.size,
+                    Fund_size_and_hold_rate.institution_holds, 
+                    Fund_size_and_hold_rate.datetime
+                ).filter(
+                    Fund_size_and_hold_rate.fund_id.in_(self.dts.fund_list),
+                    Fund_size_and_hold_rate.datetime >= self.start_date - datetime.timedelta(days = 200),
+                    Fund_size_and_hold_rate.datetime <= self.end_date,
+                )
+            _size_and_hold_rate = pd.read_sql(_fund_size_and_hold_rate_query.statement, _fund_size_and_hold_rate_query.session.bind)
+            self.dts.fund_size = _size_and_hold_rate[['fund_id','size','datetime']].pivot_table(index='datetime', columns='fund_id', values='size')
+            self.dts.fund_com_hold = _size_and_hold_rate[['fund_id','institution_holds','datetime']].pivot_table(index='datetime', columns='fund_id', values='institution_holds')
+            _dt_index = self.dts.trading_days[self.dts.trading_days.datetime >= (self.start_date - datetime.timedelta(days = 200))].datetime
+            self.dts.fund_size = self.dts.fund_size.reindex(_dt_index).fillna(method='ffill')
+            self.dts.fund_com_hold = self.dts.fund_com_hold.reindex(_dt_index).fillna(method='ffill')
+
             if print_time:
                 print(f'\t[time][basic] fetch fund info : {_tm_basic_fetch_info - _tm_basic_start}')
                 print(f'\t[time][basic] filter fund info: {_tm_basic_filter_info - _tm_basic_fetch_info}')
@@ -94,7 +121,6 @@ class FundDataManager(DataManager):
         # get derived data first
         with DerivedDatabaseConnector().managed_session() as derived_session:
             _tm_deriv_start = time.time()
-            # fund indicator
             _fund_indicator_query = derived_session.query(
                     FundIndicator.fund_id,
                     FundIndicator.datetime,
@@ -102,6 +128,8 @@ class FundDataManager(DataManager):
                     FundIndicator.beta,
                     FundIndicator.fee_rate,
                     FundIndicator.track_err,
+                    FundIndicator.year_length,
+                    FundIndicator.timespan,
                 ).filter(
                     FundIndicator.fund_id.in_(self.dts.fund_list),
                     FundIndicator.datetime >= self.start_date,
@@ -145,8 +173,7 @@ class FundDataManager(DataManager):
         self._is_inited = True
         self._score_manager.set_dts(self.dts)
         if score_pre_calc:
-            self._score_manager.pre_calculate()
-            self._score_manager.remove_a_score_if_c_exsit(self.dts.fund_info)
+            self._score_manager.pre_calculate(is_filter_c=True)
         _tm_finish = time.time()
         print(self.dts)
         if print_time:
@@ -175,11 +202,11 @@ class FundDataManager(DataManager):
             return pd.DataFrame()
         return self.dts.index_pct.loc[dt]
 
-    def get_fund_score(self, dt, index_id) -> dict: 
-        return self._score_manager.get_fund_score(dt, index_id)
+    def get_fund_score(self, dt, index_id, is_filter_c=True) -> dict: 
+        return self._score_manager.get_fund_score(dt, index_id, is_filter_c)
 
-    def get_fund_scores(self, dt) -> dict:
-        return self._score_manager.get_fund_scores(dt, self.dts.index_list)
+    def get_fund_scores(self, dt, is_filter_c=True) -> dict:
+        return self._score_manager.get_fund_scores(dt, self.dts.index_list, is_filter_c)
 
     def get_fund_nav(self, dt):
         df = self.dts.fund_nav
