@@ -198,9 +198,9 @@ class AssetTradeRateItem(AssetDict):
 class AssetTradeParam:
 
     MinCountAmtDiff: float=0.001 # 少于总资产千分之一的资产Amount差异，不计在内
-    MinActionAmtDiff: float=0.03 # 如果没有资产的偏离达到这个比例，那就不调仓
-    EnableCommission: bool=False # 是否开启交易手续费计算
-    PurchaseDiscount: float=1.0  # 大类资产的买入折扣
+    MinActionAmtDiff: float=0.06 # 如果没有资产的偏离达到这个比例，那就不调仓
+    EnableCommission: bool=True # 是否开启交易手续费计算
+    PurchaseDiscount: float=0.15  # 大类资产的买入折扣
     RedeemDiscount: float=1.0    # 大类资产的卖出折扣
     AssetPurchaseRate: dict=None 
     AssetRedeemRate: dict=None
@@ -226,8 +226,8 @@ class FundTradeParam:
     MaxFundNumUnderAsset: float=3 # 同一个大类资产下基金最多数
     MinCountAmtDiff: float=0.0003 # 少于总资产万分之三的资产Amount差异，不做基金调仓处理
     MinActionAmtDiff: float=0.03 # 如果没有资产的基金市值偏离达到这个比例，那就不调仓
-    EnableCommission: bool=False # 是否开启交易手续费计算
-    PurchaseDiscount: float=1.0  # 基金的买入折扣
+    EnableCommission: bool=True # 是否开启交易手续费计算
+    PurchaseDiscount: float=0.15 # 基金的买入折扣
     RedeemDiscount: float=1.0    # 基金的卖出折扣
 
 @dataclasses.dataclass
@@ -240,7 +240,7 @@ class TAAParam:
     HighStop:     float=0.5     # 进入估值偏高区后，什么时候算作退出
     LowStop:      float=0.5     # 进入估值偏低区后，什么时候算作退出
     TuneCash:     bool=False    # 默认taa调仓时，不调整cash
-    ToMmf:        bool=False    # 默认taa加减仓不改变货基权重
+    ToMmf:        bool=True    # 默认taa加减仓不改变货基权重
 
 @dataclasses.dataclass
 class FundScoreParam:
@@ -374,8 +374,11 @@ class FundPosition:
         if pos_item.update(trade):
             p = trade.trade_date if trade.trade_price else trade.mark_price
             amt = trade.amount if trade.amount else trade.volume * p
-            self.cash += (-amt if trade.is_buy else amt) - trade.commission
-            assert self.cash >= -1e-8, f"fund backtest cash should not be negative! (cash){self.cash} (trade){trade}"
+            cash_left = self.cash + (-amt if trade.is_buy else amt) - trade.commission 
+            if cash_left <= -1e-8:
+                print(f'cash negative alert: cancel order due to cash negative trade: {trade}, cash left if trade successed: {cash_left}')
+                return False
+            self.cash = cash_left
             return True
         else:
             return False
@@ -459,6 +462,27 @@ class TaaTunerParam:
         'sp500rmb':'pe_ttm',
     }
 
+@dataclasses.dataclass
+class ScoreFilter:
+
+    FilterIndexId:list = None
+    CompanyHoldLimit:int = 95
+    SizeBottom:int = 1e8
+    SizeTop:int = 1e10
+    
+    def __post_init__(self):
+        self.FilterIndexId = self.FilterIndexId or ['national_debt']
+
+@dataclasses.dataclass
+class ScorePenaltyParams:
+
+    FilterYearIndex:list = None
+    Penalty:float = -0.5 
+    JudgeYearLength:float = 1.0
+
+    def __post_init__(self):
+        self.FilterYearIndex = self.FilterYearIndex or ['national_debt', 'csi500']
+
 if __name__ == '__main__':
     a = AssetWeight(hs300=0.1, csi500=0.1, cash=0.8)
     print(a)
\ No newline at end of file
