@@ -1,4 +1,4 @@
-from sqlalchemy import CHAR, Column, Integer, Text, Index, BOOLEAN, text
+from sqlalchemy import CHAR, Column, Integer, Index, BOOLEAN, text, TEXT
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.dialects.mysql import DOUBLE, DATE, TINYINT, DATETIME
 
@@ -23,7 +23,8 @@ class IndexInfo(Base):
     industry_tag = Column(CHAR(64)) # 行业标签
     tag_method = Column(CHAR(64)) # 估值评分采用方法
     desc_name = Column(CHAR(64)) # 名称
-    em_id = Column(CHAR(20)) #东财代码
+    em_id = Column(CHAR(20)) # ChoiceID
+    em_plate_id = Column(CHAR(20)) # Choice板块ID
 
 
 class StockInfo(Base):
@@ -57,7 +58,7 @@ class FundInfo(Base):
     end_date = Column(DATE) # 关闭日期
     wind_class_1 = Column(CHAR(64)) # Wind基金类型
     wind_class_2 = Column(CHAR(64)) # Wind基金二级类型
-    manager_id = Column(Text) # 基金经理
+    manager_id = Column(TEXT) # 基金经理
     company_id = Column(CHAR(64)) # 基金公司
     benchmark = Column(CHAR(255)) # 业绩基准
     full_name = Column(CHAR(255)) # 基金全名
@@ -84,7 +85,8 @@ class FundInfo(Base):
     is_a = Column("is_a", BOOLEAN, nullable=False)  #是否是etf  0 非  1 a   
     ac_filter = Column("ac_filter", BOOLEAN, nullable=False) #是否ac均有要排除的a 0 排除  1 保留  
     is_selected_mmf = Column(TINYINT(1)) # 是否基金规模最大的前50只货币基金(10年前有两只存在)  0 非  1 是
-    
+    national_debt_extension = Column(TINYINT(1)) # 纯债tag  0 非  1 是
+
 class TradingDayList(Base):
     '''交易日列表'''
 
@@ -193,3 +195,35 @@ class FundRatingLatest(Base):
     __table_args__ = (
         Index('idx_fund_rating_latest_datetime', 'update_time'),
     )
+
+
+class StyleAnalysisStockFactor(Base):
+    '''风格分析股票因子'''
+
+    __tablename__ = 'style_analysis_stock_factor'
+
+    stock_id = Column(CHAR(10), primary_key=True)  # EM股票ID
+    datetime = Column(DATE, primary_key=True)  # 日期
+    rate_of_return = Column(DOUBLE(asdecimal=False), nullable=False)  # 收益率
+    latest_size = Column(DOUBLE(asdecimal=False))  # 规模
+    bp = Column(DOUBLE(asdecimal=False))  # 价值
+    short_term_momentum = Column(DOUBLE(asdecimal=False))  # 短期动量
+    long_term_momentum = Column(DOUBLE(asdecimal=False))  # 长期动量
+    high_low = Column(DOUBLE(asdecimal=False))  # 波动率
+
+    __table_args__ = (
+        Index('idx_style_analysis_stock_factor_datetime', 'datetime'),
+    )
+
+class Fund_size_and_hold_rate(Base):
+    '''基金规模和持有人比例'''
+
+    __tablename__ = 'fund_size_and_hold_rate'
+
+    fund_id = Column(CHAR(10), primary_key=True)  # 基金id
+    datetime = Column(DATE, primary_key=True)  # 日期
+    size = Column(DOUBLE(asdecimal=False)) # 最新规模
+    personal_holds = Column(DOUBLE(asdecimal=False)) # 个人持有比例 单位百分比 
+    institution_holds = Column(DOUBLE(asdecimal=False)) # 机构持有比例 单位百分比 
+
+    
\ No newline at end of file
