@@ -140,18 +140,20 @@ class FundIndicator(Base):
     track_err = Column(DOUBLE(asdecimal=False)) # 跟踪误差
     timespan = Column(DOUBLE(asdecimal=False)) # 历史数据跨度(年)
     fee_rate = Column(DOUBLE(asdecimal=False)) # 费率
-    # treynor = Column(DOUBLE(asdecimal=False)) # 特雷诺比率
-    # mdd = Column(DOUBLE(asdecimal=False)) #净值最大回撤
-    # down_risk = Column(DOUBLE(asdecimal=False)) #下行风险
-    # ret_over_period = Column(DOUBLE(asdecimal=False)) #区间收益率
-    # annual_avg_daily_ret = Column(DOUBLE(asdecimal=False)) #年化日均收益
-    # vol = Column(DOUBLE(asdecimal=False)) #波动率
-    # m_square = Column(DOUBLE(asdecimal=False)) #M平方测度 风险调整收益指标
-    # time_ret = Column(DOUBLE(asdecimal=False)) #择时收益
-    # var = Column(DOUBLE(asdecimal=False)) #资产在险值
-    # r_square = Column(DOUBLE(asdecimal=False)) #决定系数R方
-    # sharpe = Column(DOUBLE(asdecimal=False)) #夏普率
-    # tm_coef = Column(DOUBLE(asdecimal=False)) #风险收益指标
+    info_ratio = Column(DOUBLE(asdecimal=False)) # 信息比率
+    treynor = Column(DOUBLE(asdecimal=False)) # 特雷诺比率
+    mdd = Column(DOUBLE(asdecimal=False)) #净值最大回撤
+    down_risk = Column(DOUBLE(asdecimal=False)) #下行风险
+    ret_over_period = Column(DOUBLE(asdecimal=False)) #区间收益率
+    annual_avg_daily_ret = Column(DOUBLE(asdecimal=False)) #年化日均收益
+    annual_vol = Column(DOUBLE(asdecimal=False)) #年化波动率
+    annual_ret = Column(DOUBLE(asdecimal=False)) #年化收益
+    m_square = Column(DOUBLE(asdecimal=False)) #M平方测度 风险调整收益指标
+    time_ret = Column(DOUBLE(asdecimal=False)) #择时收益
+    var = Column(DOUBLE(asdecimal=False)) #资产在险值
+    r_square = Column(DOUBLE(asdecimal=False)) #决定系数R方
+    sharpe = Column(DOUBLE(asdecimal=False)) #夏普率
+    year_length = Column(DOUBLE(asdecimal=False)) #成立年限
 
     __table_args__ = (
         Index('idx_fund_indicator_datetime', 'datetime'),
@@ -207,3 +209,36 @@ class AssetAllocationInfo(Base):
     mdd_d2 = Column(DATE) # 最大回撤结束时间
     start_date = Column(DATE) # 回测开始时间
     end_date = Column(DATE) # 回测结束时间
+
+class StyleAnalysisFactorReturn(Base):
+    '''风格分析因子收益'''
+
+    __tablename__ = 'style_analysis_factor_return'
+
+    universe_index = Column(CHAR(20), primary_key=True)  # universe, 全市场股票时值为all
+    datetime = Column(DATE, primary_key=True)  # 日期
+    latest_size = Column(DOUBLE(asdecimal=False))  # 规模
+    bp = Column(DOUBLE(asdecimal=False))  # 价值
+    short_term_momentum = Column(DOUBLE(asdecimal=False))  # 短期动量
+    long_term_momentum = Column(DOUBLE(asdecimal=False))  # 长期动量
+    high_low = Column(DOUBLE(asdecimal=False))  # 波动率
+    const = Column(DOUBLE(asdecimal=False))  # 常数项
+
+class AllocationDistribution(Base):
+    '''有效前沿下各mdd最优策略'''
+
+    __tablename__ = 'allocation_distribution'
+
+    ef_id = Column(CHAR(8), primary_key=True)  # 有效前沿id
+    version = Column(Integer, primary_key=True) # 版本号
+    annual_ret = Column(DOUBLE(asdecimal=False)) #年化收益
+    csi500 = Column(DOUBLE(asdecimal=False)) # 中证500权重
+    gem = Column(DOUBLE(asdecimal=False)) #创业板权重
+    gold = Column(DOUBLE(asdecimal=False)) #黄金权重
+    hs300 = Column(DOUBLE(asdecimal=False)) #沪深300权重
+    mdd = Column(DOUBLE(asdecimal=False)) # 最大回撤
+    mdd_up_limit = Column(DOUBLE(asdecimal=False)) #最大回测档位上限
+    mmf = Column(DOUBLE(asdecimal=False)) # 货币基金权重
+    national_debt = Column(DOUBLE(asdecimal=False)) #国债权重
+    sp500rmb = Column(DOUBLE(asdecimal=False)) #标普500权重
+    cash = Column(DOUBLE(asdecimal=False)) #现金权重
\ No newline at end of file
