@@ -444,6 +444,9 @@ class EmStockPrice(Base):
     BUYVOL = Column('buy_vol', DOUBLE(asdecimal=False), server_default=text('0')) # 内盘成交量
     SELLVOL = Column('sell_vol', DOUBLE(asdecimal=False), server_default=text('0')) # 外盘成交量
 
+    __table_args__ = (
+        Index('idx_em_stock_price_datetime', 'datetime'),
+    )
 
 class EmStockPostPrice(Base):
     '''Choice股票后复权数据表'''
@@ -462,7 +465,11 @@ class EmStockPostPrice(Base):
     VOLUME = Column('volume', DOUBLE(asdecimal=False), server_default=text('0')) # 成交量
     TRADESTATUS = Column('trade_status', CHAR(20), nullable=False) # 交易状态
     TAFACTOR = Column('tafactor', DOUBLE(asdecimal=False), server_default=text("0")) # 复权因子（后）
-    
+
+    __table_args__ = (
+        Index('idx_em_stock_post_price_datetime', 'datetime'),
+    )
+
 class EmStockDailyInfo(Base):
     '''Choice股票每日更新'''
 
@@ -480,6 +487,9 @@ class EmStockDailyInfo(Base):
     AHOLDER = Column('a_holder', TEXT)  # 实际控制人
     ESTPEG = Column('est_peg', DOUBLE(asdecimal=False)) # 预测PEG(最近一年)
 
+    __table_args__ = (
+        Index('idx_em_stock_daily_info_datetime', 'datetime'),
+    )
 
 class EmStockInfo(Base):
     '''Choice股票信息表'''
@@ -586,6 +596,7 @@ class EmStockFinFac(Base):
     BALANCESTATEMENT_103 = Column('balance_statement_103', DOUBLE(asdecimal=False)) # 非流动负债合计
     BALANCESTATEMENT_141 = Column('balance_statement_141', DOUBLE(asdecimal=False)) # 股东权益合计
     BALANCESTATEMENT_140 = Column('balance_statement_140', DOUBLE(asdecimal=False)) # 归属于母公司股东权益合计
+    BALANCESTATEMENT_195 = Column('balance_statement_195', DOUBLE(asdecimal=False)) # 其他权益工具
     INCOMESTATEMENT_9 = Column('income_statement_9', DOUBLE(asdecimal=False)) # 营业收入
     INCOMESTATEMENT_48 = Column('income_statement_48', DOUBLE(asdecimal=False)) # 营业利润
     INCOMESTATEMENT_60 = Column('income_statement_60', DOUBLE(asdecimal=False)) # 净利润
@@ -599,6 +610,10 @@ class EmStockFinFac(Base):
     CASHFLOWSTATEMENT_82 = Column('cashflow_statement_82', DOUBLE(asdecimal=False)) # 现金及现金等价物净增加额
     CASHFLOWSTATEMENT_86 = Column('cashflow_statement_86', DOUBLE(asdecimal=False)) # 资产减值准备
 
+    __table_args__ = (
+        Index('idx_em_stock_fin_fac_datetime', 'datetime'),
+    )
+
 class EmStockRefinancing(Base):
     '''Choice股票再融资信息表'''
 
@@ -634,3 +649,24 @@ class EmTradeDates(Base):
 
     __tablename__ = 'em_tradedates'
     TRADEDATES = Column(DATE, primary_key=True)
+
+class EmIndexComponent(Base):
+    '''Choice指数成分'''
+
+    __tablename__ = 'em_index_component'
+
+    index_id = Column(CHAR(20), primary_key=True)
+    datetime = Column(DATE, primary_key=True) # 日期
+    em_id = Column(CHAR(20), nullable=False)  # Choice ID
+    em_plate_id = Column(CHAR(20), nullable=False)  # Choice板块ID
+    stock_list = Column(TEXT, nullable=False)  # 股票列表
+
+class EmFundHoldingRate(Base):
+    '''Choice基金持有人比例'''
+    __tablename__ = 'em_fund_holding_rate'
+
+    CODES =Column(CHAR(10), primary_key=True) # 合约代码
+    DATES = Column(DATE, primary_key=True) # 日期
+    HOLDPERSONALHOLDINGPCT = Column(DOUBLE(asdecimal=False)) # 个人持有比例 单位百分比
+    HOLDINSTIHOLDINGPCT = Column(DOUBLE(asdecimal=False)) # 机构持有比例 单位百分比
+
