@@ -17,7 +17,7 @@ else:
 class AssetPainter(object):
 
     @staticmethod
-    def plot_asset_weights(asset_weight_dic:dict):
+    def plot_asset_weights(asset_weight_dic:dict, saa:dict):
         asset_weight_df = pd.DataFrame(asset_weight_dic).T
         res = []
         for dic, s in zip(asset_weight_df.to_dict('records'), asset_weight_df.sum(axis = 1).values):
@@ -27,14 +27,29 @@ class AssetPainter(object):
         cols = [col for col in cols if df[col].sum() != 0]
         df = df[cols]
         df.index = asset_weight_df.index
-        df.plot.area(figsize=(18,9),legend=False,fontsize = 13)
-        l = pl.legend(loc='lower left',fontsize = 9)
-        s = pl.title('asset weights history', fontsize=20)
+        l = df.columns
+        core_assets = ['hs300','csi500','gem','sp500rmb']
+        asset_in_c = [i for i in l if i in core_assets]
+        asset_not_in_c = sorted([i for i in l if i not in core_assets and i != 'cash']) 
+        if 'national_debt' in asset_not_in_c:
+            asset_not_in_c.remove('national_debt')
+            asset_not_in_c = ['national_debt'] + asset_not_in_c
+        l = asset_in_c + asset_not_in_c + ['cash']
+        df = df[l]
+        df.plot.area(figsize=(18,9),legend=False,fontsize = 17)
+        plt.legend(loc='lower left',fontsize = 17)
+        plt.title('asset weights history', fontsize=20)
+        st = [k+':'+str(round(v,2)) for k, v in saa.items() if v > 0]
+        st = ' '.join(st)
+        plt.suptitle(st,y=0.87,fontsize=17)
+        plt.grid()
+        plt.show()
 
     @staticmethod
-    def plot_asset_mdd_period(asset_mv:pd.DataFrame, saa_weight:dict, index_price: pd.DataFrame):
+    def plot_asset_mdd_period(asset_mv:pd.DataFrame, saa_weight:dict, index_price: pd.DataFrame, asset_weight:dict):
         df = asset_mv
         mdd_part1 = (df.loc[:, 'mv'] / df.loc[:, 'mv'].rolling(10000, min_periods=1).max())
+        mdd = round(1 - mdd_part1.min(),4)
         mdd_date1 = df.loc[:mdd_part1.idxmin(),'mv'].idxmax()
         mdd_date2 = mdd_part1.idxmin()
         l = [k for k, v in saa_weight.items() if v > 0]
@@ -43,9 +58,41 @@ class AssetPainter(object):
         if df.empty:
             return 
         df = df / df.iloc[0]
-        df.plot.line(figsize=(18,9),legend=False,fontsize = 13)
-        l = pl.legend(loc='lower left',fontsize = 13)
-        s = pl.title('asset bt during mdd period  price of holding assets', fontsize=20)    
+        asset_weight = asset_weight[mdd_date1]
+        for col in df:
+            df[col] = 1 - (1 - df[col].values )* asset_weight[col]
+        d1 = mdd_date1.strftime('%Y-%m-%d')
+        d2 = mdd_date2.strftime('%Y-%m-%d')
+        df.plot.line(figsize=(18,9),legend=False,fontsize = 17)
+        l = pl.legend(loc='lower left',fontsize = 17)
+        s = pl.title('asset bt during mdd period price of holding assets', fontsize=20)
+        plt.suptitle(f'mdd : {mdd}, from {d1} to {d2}',y=0.87,fontsize=17)    
+        plt.grid()
+        plt.show()
+
+    @staticmethod
+    def plot_asset_mdd_amount(asset_mv:pd.DataFrame, index_price: pd.DataFrame, position_history:dict):    
+        index_ret = index_price / index_price.shift(1) - 1
+        df = asset_mv
+        mdd_part1 = (df.loc[:, 'mv'] / df.loc[:, 'mv'].rolling(10000, min_periods=1).max())
+        mdd = round(1 - mdd_part1.min(),4)
+        mdd_date1 = df.loc[:mdd_part1.idxmin(),'mv'].idxmax()
+        mdd_date2 = mdd_part1.idxmin()
+        volume_df = pd.DataFrame(position_history).T
+        volume_df = volume_df.loc[mdd_date1: mdd_date2]
+        if volume_df.empty:
+            return 
+        amount_df = volume_df.drop(['cash'], axis=1) * index_price.loc[mdd_date1: mdd_date2] * index_ret.loc[mdd_date1: mdd_date2]
+        select_cols = []
+        for col in amount_df:
+            if amount_df[col].sum() != 0:
+                select_cols.append(col)
+        amount_df = amount_df[select_cols].cumsum()
+        amount_df.plot.line(figsize=(18,9))
+        l = pl.legend(loc='lower left',fontsize = 17)
+        s = pl.title('asset bt during mdd period money loss', fontsize=20)
+        plt.grid()
+        plt.show()
 
     @staticmethod
     def plot_asset_taa_saa(saa_mv:pd.DataFrame, taa_mv:pd.DataFrame, index_id:str, index_pct:pd.DataFrame):
@@ -54,9 +101,11 @@ class AssetPainter(object):
         begin_d = index_pct[[index_id]].index.values[0]
         df1 = saa_mv[['saa']].join(taa_mv['taa'])
         df1 = (df1 / df1.iloc[0])
-        df1.plot.line(figsize=(20,12),legend=False,fontsize = 13)
+        df1.plot.line(figsize=(20,12),legend=False,fontsize = 17)
         l = pl.legend(loc='upper left',fontsize = 25)
         s = pl.title(f'{index_id} taa', fontsize=20)
+        plt.grid()
+        plt.show()
 
     @staticmethod
     def plot_taa_analysis(saa_mv:pd.DataFrame, taa_mv:pd.DataFrame, index_id:str, index_pct:pd.DataFrame, taa_params:TAAParam, index_price:pd.DataFrame):
@@ -171,6 +220,7 @@ class AssetPainter(object):
             _plot_pct_high(ax, high_i, df, price_col)
         for low_i in low_plus_periods:
             _plot_pct_low(ax, low_i, df, price_col)
+        plt.grid()
         plt.show()
 
         if taa_mv is not None and saa_mv is not None:
@@ -187,6 +237,7 @@ class AssetPainter(object):
                 _plot_dif(ax, high_i, df, False)
             for low_i in low_plus_periods:
                 _plot_dif(ax, low_i, df, True)
+            pl.grid()
             plt.show()
 
     @staticmethod
@@ -204,3 +255,8 @@ class AssetPainter(object):
         t = plt.title(f'market value with AmtDiff {m} annual_ret {annual_ret} mdd {mdd} ret_over_mdd {ret_over_mdd} ', fontsize=20)
         for d, r in asset_mv.loc[rebalance_date,:].iterrows():
             pl.scatter(x=d, y=r.mv, c='r',s=50)
+        pl.grid()
+        pl.show()
+
+
+        
\ No newline at end of file
