@@ -17,7 +17,7 @@ else:
 class HybridPainter(object):
 
     @staticmethod
-    def plot_market_value(mv_df:pd.DataFrame, backtest_type:str, index_price:pd.DataFrame, saa_weight:dict):
+    def plot_market_value(mv_df:pd.DataFrame, backtest_type:str, index_price:pd.DataFrame, saa_weight:dict, bk_stats:dict):
         index_df_raw = index_price.loc[mv_df.index[0]:mv_df.index[-1],]
         index_df = index_df_raw.copy().fillna(method='bfill')
         index_df = index_df / index_df.iloc[0]
@@ -35,9 +35,14 @@ class HybridPainter(object):
         wgts_df = pd.DataFrame(w_l).set_index('datetime')
         mv_df['benchmark'] = (wgts_df * index_df).sum(axis = 1)
         mv_df = mv_df / mv_df.iloc[0]
-        mv_df.plot.line(figsize=(20,12),legend=False,fontsize = 13)
-        l = pl.legend(loc='lower left',fontsize = 10)
+        mv_df.plot.line(figsize=(20,12),legend=False,fontsize = 17)
+        l = pl.legend(loc='lower left',fontsize = 17)
         s = pl.title(f'{backtest_type} market value', fontsize=20)
+        ar = round(bk_stats['annual_ret'],4)
+        mdd = round(bk_stats['mdd'], 4)
+        sharpe = round(bk_stats['sharpe'],4)
+        pl.suptitle(f'annual_ret : {ar} mdd : {mdd} sharpe : {sharpe}',y=0.87,fontsize=17)
+        plt.grid()
 
     @staticmethod
     def plot_asset_fund_mv_diff(asset_mv:pd.DataFrame, fund_mv:pd.DataFrame):
@@ -48,10 +53,11 @@ class HybridPainter(object):
         check_diff = fund_mv.join(asset_mv)
         check_diff = check_diff / check_diff.iloc[0]
         check_diff['diff'] = 100 * (check_diff['fund_mv']  - check_diff['asset_mv']) / check_diff['asset_mv'] 
-        check_diff[['fund_mv','asset_mv']].plot.line(figsize=(20,12),legend=False,fontsize = 13)
-        l = pl.legend(loc='lower left',fontsize = 10)
+        check_diff[['fund_mv','asset_mv']].plot.line(figsize=(20,12),legend=False,fontsize = 17)
+        l = pl.legend(loc='lower left',fontsize = 17)
         s = pl.title('asset and fund market value ', fontsize=20)
-        check_diff[['diff']].plot.line(figsize=(20,12),legend=False,fontsize = 13)
-        l = pl.legend(loc='lower left',fontsize = 10)
+        plt.grid()
+        check_diff[['diff']].plot.line(figsize=(20,12),legend=False,fontsize = 17)
+        l = pl.legend(loc='lower left',fontsize = 17)
         s = pl.title('asset and fund market value diff % ', fontsize=20)
-        
\ No newline at end of file
+        plt.grid()
\ No newline at end of file
