@@ -80,10 +80,10 @@ class ReportHelper:
         self.asset_position_history[dt] = asset_cur_position.__dict__##
         self.pending_trade_history[dt] = pend_trades##
         if fund_cur_position is not None:
-            dic = {f : f_info.__dict__  for f, f_info in fund_cur_position.funds.items()}
+            dic = {f : f_info.__dict__  for f, f_info in fund_cur_position.copy().funds.items()}
             for f, f_info in dic.items():
                 f_info['price'] =  fund_nav[f]
-            self.fund_position_history[dt] = dic##        
+            self.fund_position_history[dt] = dic     
             self.fund_cash_history[dt] = fund_cur_position.cash
             mv, fund_w = fund_cur_position.calc_mv_n_w(fund_navs=fund_nav)
             self.fund_marktet_price_history[dt] = mv
@@ -158,6 +158,15 @@ class ReportHelper:
         w['last_increase_rate'] = res.last_increase_rate
         w['market_value'] = df #dataframe
         w['rebalance_times'] = len(self.rebalance_date)
+        hold_days = self.rebalance_date + [df.index.values[-1]]
+        index_list = df.index.tolist()
+        result = {}
+        for d in hold_days[:-1]:
+            d1 = hold_days[hold_days.index(d) + 1]
+            idx0 = index_list.index(d)
+            idx1 = index_list.index(d1)
+            result[d] = (idx1 - idx0) / 242
+        w['hold_years'] = result # dic
         return w
 
     def get_asset_stat(self):
@@ -189,6 +198,19 @@ class ReportHelper:
         pos_df = pos_df.sort_values('weight', ascending = False)
         return pos_df
 
+    def get_last_target_fund_allocation(self, target_fund):
+        res = []
+        fund_info_dic = self.fund_info[['fund_id','desc_name']].set_index('fund_id').to_dict()['desc_name']
+        for fund_id, fund_w in target_fund.items():
+            dic = {'fund_id':fund_w.fund_id,
+                'index_id':fund_w.index_id, 
+                'weight':round(fund_w.fund_wgt,3),
+                'desc_name':fund_info_dic[fund_w.fund_id]}
+            res.append(dic)
+        target_fund = pd.DataFrame(res).set_index('fund_id')
+        target_fund.index.name = list(self.fund_position_history.keys())[-1]
+        return target_fund.sort_values('weight', ascending = False)[['index_id','weight','desc_name']]
+
     def get_fund_trade(self):
         fsdf = self.fund_score.copy()
         fund_info_df = self.fund_info.copy().set_index('fund_id')
@@ -262,27 +284,34 @@ class ReportHelper:
         w = self.get_asset_stat()
         del w['market_value']
         pprint(w)
-        AssetPainter.plot_asset_weights(self.asset_weights_history)
-        HybridPainter.plot_market_value(self.asset_mv, bt_type, self.index_price, self.saa_weight)
-        AssetPainter.plot_asset_mdd_period(self.asset_mv, self.saa_weight, self.index_price)
+        AssetPainter.plot_asset_weights(self.asset_weights_history, self.saa_weight)
+        HybridPainter.plot_market_value(self.asset_mv, bt_type, self.index_price, self.saa_weight, w)
+        AssetPainter.plot_asset_mdd_period(self.asset_mv, self.saa_weight, self.index_price, self.asset_weights_history)
+        AssetPainter.plot_asset_mdd_amount(self.asset_mv, self.index_price, self.asset_position_history)
 
     def backtest_fund_plot(self):
         bt_type = 'fund'
         print(f'{bt_type} report')
         w = self.get_fund_stat()
+        mv = w['market_value'].copy()
         del w['market_value']
         pprint(w)
+        w['market_value'] = mv
         self.get_index_fee_avg()
-        FundPainter.plot_fund_weights(self.fund_weights_history, self.fund_cash_history, self.fund_marktet_price_history)
-        HybridPainter.plot_market_value(self.fund_mv, bt_type, self.index_price, self.saa_weight)
+        FundPainter.plot_fund_weights(self.fund_weights_history, self.fund_cash_history, self.fund_marktet_price_history, self.saa_weight)
+        HybridPainter.plot_market_value(self.fund_mv, bt_type, self.index_price, self.saa_weight, w)
         FundPainter.plot_fund_mdd_periods(self.fund_mv, self.fund_weights_history, self.fund_nav, self.fund_info)
-        HybridPainter.plot_asset_fund_mv_diff(self.asset_mv, self.fund_mv)
+        FundPainter.plot_fund_mdd_amounts(self.fund_mv, self.fund_nav, self.fund_info, self.fund_position_history.copy())
+        FundPainter.plot_fund_alpha(self.fund_nav, self.fund_info, self.index_price, self.fund_position_history.copy(), w)
+        FundPainter.plot_mv_on_each_asset(self.fund_nav, self.fund_info, self.index_price, self.fund_position_history.copy(), w)
+    
+        #HybridPainter.plot_asset_fund_mv_diff(self.asset_mv, self.fund_mv)
         if not self.index_fee.empty:
             FundPainter.plot_index_fund_fee(self.index_fee)
         if not self.turnover_df.empty:
             FundPainter.plot_fund_ret_each_year(self.turnover_df, w['mdd'], w['annual_ret'] )
             FundPainter.plot_turnover_rate_each_year(self.turnover_df, self.turnover_rate_yearly_avg)
-
+ 
     def plot_whole(self):
         self.backtest_fund_plot()
         self.backtest_asset_plot()
@@ -339,4 +368,35 @@ class ReportHelper:
         df_diff_part['end_date'] = next_date
         df_result = df_diff_part[df_diff_part[index_id] != TAAStatusMode.NORMAL][['begin_date','end_date',index_id]]
         df_result = df_result.rename(columns = {index_id:'status'})
-        return df_result.to_dict('records') 
\ No newline at end of file
+        return df_result.to_dict('records') 
+
+    def get_date_score(self, datetime:datetime.date):
+        fund_score = self.fund_score[datetime]
+        fund_score_raw = self.fund_score_raw[datetime]
+        desc_name_dic = self.fund_info[['fund_id','desc_name']].set_index('fund_id').to_dict()['desc_name']
+        res = []
+        for index_id, score_dic in fund_score.items():
+            for fund_id, fund_s in score_dic.items():
+                dic = {
+                    'index_id' : index_id,
+                    'fund_id': fund_id,
+                    'desc_name':desc_name_dic[fund_id],
+                    'score':fund_s,
+                    'raw_score':fund_score_raw[index_id][fund_id],
+                }
+                res.append(dic)
+        return pd.DataFrame(res).sort_values(['index_id','score'],ascending=[True,False]).set_index('index_id')
+
+    @staticmethod
+    def plot_fund_score(fund_id:str='200002!0',begin_date:datetime.date=datetime.date(2015,1,1),dm:FundDataManager=None):
+        FundPainter.plot_fund_rank(score_dic=dm._score_manager.score_cache,
+                                   fund_info=dm.dts.fund_info,
+                                   fund_id=fund_id,
+                                   begin_date=begin_date,
+                                   fund_index_map=dm.dts.fund_index_map)
+
+
+
+if __name__ == "__main__":
+    f._report_helper.plot_fund_score(dm=m)
+    
\ No newline at end of file
