@@ -1,14 +1,18 @@
 import time
 import datetime
 import pandas as pd
+import numpy as np
 from pprint import pprint 
 import operator
 import traceback
+import concurrent.futures
+import os
 from .backtest import FundBacktestEngine
 from .trader import AssetTrader
 from ...data.manager.manager_fund import FundDataManager
 from ...data.struct import AssetTrade, FundTrade, FundScoreParam, TAAParam, TaaTunerParam, AssetTradeParam, AssetWeight
 from ...data.api.derived import DerivedDataApi
+from ...data.manager.score import ScoreFunc
 
 class TaaTuner:
 
@@ -390,6 +394,118 @@ class SaaTuner:
         result['rebalance_times'] = [len(_) for _ in result['rebalance_date']]
         return result
 
+class ScoreTuner:
+
+    INDICATOR_SIGN={
+        'alpha':1,
+        'beta':-1,
+        'fee_rate':-1,
+        'track_err':-1,
+    }
+
+    def __init__(self, dm:FundDataManager, index_id:str='national_debt', folder:str='good_luck'):
+        self.dm = dm
+        self.index_id = index_id
+        self.folder = folder
+        self.search_datelist = self.dm.dts.index_fund_indicator_pack.loc[self.index_id].index.levels[0].tolist()
+        self.score_list = self.make_score_list()
+        self.index_indicator = self.dm.dts.index_fund_indicator_pack.loc[self.index_id]
+        time_span = list(set(self.index_indicator.timespan))[0]
+        self.index_indicator = self.index_indicator[self.index_indicator.year_length > time_span]
+
+    def make_date_param_list(self, time_span:int=242, begin_date=datetime.date(2015,1,1)):
+        search_datelist = [_ for _ in self.search_datelist if _ >= begin_date]
+        param_list = []
+        for idx,d in enumerate(search_datelist):
+            end_idx = time_span + idx
+            if end_idx < len(search_datelist):
+                param_list.append({
+                    'begin_date':d,
+                    'end_date':search_datelist[end_idx]
+                })
+        return param_list
+
+    def make_score_list(self):
+        score_list = []
+        for alpha in np.arange(0,0.8,0.05):
+            for beta in np.arange(0,0.8,0.05):
+                for fee_rate in np.arange(0,0.5,0.05):
+                    track_err = 1 - alpha - beta - fee_rate
+                    if track_err < 0:
+                        continue
+                    score_list.append(ScoreFunc(alpha=self.INDICATOR_SIGN['alpha'] * alpha, 
+                                                beta=self.INDICATOR_SIGN['beta'] * beta, 
+                                                fee_rate=self.INDICATOR_SIGN['fee_rate'] * fee_rate,
+                                                track_err=self.INDICATOR_SIGN['track_err'] * track_err))
+        return score_list
+
+    def fund_alpha_calculator(self,top_num:int=3, 
+                                start_date:datetime.date=datetime.date(2018,1,1),
+                                end_date:datetime.date=datetime.date(2019,1,1),
+                                score_method:ScoreFunc=ScoreFunc(alpha=0.3, beta=-0.6, fee_rate=-0.1)):
+        
+        index_price = self.dm.dts.index_price.loc[start_date:end_date,[self.index_id]]
+        raw_indicator = self.index_indicator.loc[start_date]
+        normalized_indicator = raw_indicator.apply(lambda x: (x - x.mean() + 1e-6)/ (x.std() + 1e-6), axis=0)
+        normalized_indicator['beta'] = raw_indicator['beta']
+        score_raw = normalized_indicator.apply(lambda x: score_method.get(x), axis=1)
+        select_funds = pd.DataFrame(score_raw).sort_values(0,ascending=False).index[:top_num].tolist()
+        select_nav = self.dm.dts.fund_nav.loc[start_date:end_date,select_funds]
+        funds_mean_ret =  np.mean(select_nav.iloc[-1] / select_nav.iloc[0]) - 1
+        index_ret = (index_price.iloc[-1] / index_price.iloc[0]).values[0] - 1
+        return funds_mean_ret - index_ret
+
+    def score_factor_calculator(self, score_method:ScoreFunc=ScoreFunc(alpha=0.3, beta=-0.6, fee_rate=-0.1),
+                                    param_idx:int=1):
+        t1 = time.time()
+        param_list = self.make_date_param_list()
+        result = []
+        for param_i in param_list:
+            alpha_i = self.fund_alpha_calculator(top_num=3, 
+                               start_date=param_i['begin_date'], 
+                               end_date=param_i['end_date'],
+                               score_method=score_method)
+            result.append({
+                'datetime':param_i['begin_date'],
+                'alpha':alpha_i
+            })    
+        name_list = [f'{fac}_{round(wgt,2)}'for fac, wgt in score_method.__dict__.items() if wgt != 0]
+        csv_name = '~'.join(name_list)
+        csv_path = self.folder + '/' + str(param_idx) + '-' + csv_name + '.csv'
+        pd.DataFrame(result).to_csv(csv_path)
+        t2 = time.time()
+        print(f'param_idx {param_idx} finish ,cost time {t2 - t1} s')
+        return True
+        
+    def multi_search(self, score_list:list=[ScoreFunc(alpha=0.3, beta=-0.6, fee_rate=-0.1), 
+                                            ScoreFunc(alpha=0.3, beta=-0.5, fee_rate=-0.2),
+                                            ScoreFunc(alpha=0.2, beta=-0.7, fee_rate=-0.1),
+                                            ScoreFunc(alpha=0.1, beta=-0.8, fee_rate=-0.1)],
+                           cpu_num:int=2):
+        with concurrent.futures.ProcessPoolExecutor(max_workers=cpu_num) as executor:
+            future_to_bti = {executor.submit(self.score_factor_calculator, score_method, param_idx): param_idx for param_idx, score_method in enumerate(score_list)}
+            for future in concurrent.futures.as_completed(future_to_bti):
+                bt_i = future_to_bti[future]
+                try:
+                    data = future.result()
+                except Exception as exc:
+                    print(f'{bt_i} generated an exception: {exc}')
+                else:
+                    pass
+
+    def show_result(self):
+        csv_list = os.listdir(self.folder)
+        res = []
+        for csv_i in csv_list:
+            csv_path = self.folder+ csv_i
+            df_i = pd.read_csv(csv_path,index_col=0)
+            res.append({
+                'score_method':csv_i[:-4],
+                'alpha_mean':df_i.alpha.mean(),
+                'alpha_std':df_i.alpha.std(),
+            })
+        return pd.DataFrame(res)   
+
 if __name__ == "__main__":
     
     tt = TaaTunerParam()
